#ifndef __COMMANDLINE_H__
#define __COMMANDLINE_H__

#include "Typedef.h"

class FiniteAbelianGroup;

const std::string strCrossNumberOption = "/C";
const std::string strOrderOption = "/O";
const std::string strMultiplicityOption = "/M";
const std::string strFormatOption = "/F";
const std::string strScreenOption = "/D";
const std::string strInputFileOption = "/IF";
const std::string strSequenceFileOption = "/OF";
const std::string strStatsFileOption = "/OFS";
const std::string strGroupDefOption = "/G";
const std::string strLengthLowerBound = "/BL";
const std::string strLengthUpperBound = "/BU";
const std::string strForbidOrder = "/FO";
const std::string strOrderBound = "/OB";
const std::string strBlockOption = "/B";
const std::string strForbidElementOption = "/FE";
const std::string strRequireElementOption = "/RE";
const std::string strSpecialOrder = "/OS";

typedef enum {DispFormatStandard, DispFormatMultiplicity, Reserved} eDispFormat;
const std::string FormatDesc[Reserved] = {
	"Standard Format",
	"Run Length Encoded (i.e. Multiplicity Format)"
};

class CommandLineOptions
{
private:
	typedef std::vector<std::string>	OptionsList_t;

	typedef enum {InputFile, SequenceFile, StatsFile} eFileType;

	typedef std::map<int, int>	OrderBoundLookup_t;

public:

	typedef std::vector<int>	ForbiddenOrderList_t;

	typedef ForbiddenOrderList_t::const_iterator	ForbiddenOrderIter_t;


	CommandLineOptions();
	~CommandLineOptions();

	void Parse(const OptionsList_t& rOptions);

	ForbiddenOrderIter_t forbidden_order_begin() const {return m_ForbiddenOrders.begin();}
	ForbiddenOrderIter_t forbidden_order_end() const {return m_ForbiddenOrders.end();}

	bool DisplayCrossNumbers()		const {return m_bCrossNumbers;}
	bool DisplayOrders()			const {return m_bOrders;}
	bool DisplayMultiplicity()		const {return m_bMultiplicity;}
	bool DisplayToScreen()			const {return m_bDisplayScreen;}
	bool UseBlockAlgorithm()		const {return m_bBlockSearch;}
	
	int GetProgressMinChangeElmt()	const {return m_nProgressChangeElement;}
	int GetLengthLowerBound()		const {return m_nLengthLowerBound;}
	int GetLengthUpperBound()		const {return m_nLengthUpperBound;}

	int GetDesiredBlockLength()		const {return m_nBlockLength;}

	bool IsOrderBounded(int nOrder)		const;
	int GetOrderBound(int nOrder)	const;

	eDispFormat GetDispFormat()		const {return m_eDisplayFormat;}
	std::string GetInputFile()		const;
	std::string GetSequenceFile()	const;
	std::string GetStatsFile()		const;

	bool	UseInputFile() const;
	bool	UseSequenceFile() const;
	bool	UseStatsFile() const;

	FiniteAbelianGroup&	GetGroup();
	const				FiniteAbelianGroup&	GetGroup() const;
	bool				GroupExists() const;

	void				AddGroup(const std::vector<int>& values);

	std::string	GetHelpString() const;

	Set_t::const_iterator	required_begin() const	{return m_RequiredElements.begin();}
	Set_t::const_iterator	required_end() const	{return m_RequiredElements.end();}

private:
	void Initialize();

	OptionsList_t::const_iterator ParseCrossNumberOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseOrderOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseMultiplicityOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseSequenceFormatOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseScreenDisplayOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);

	OptionsList_t::const_iterator ParseInputFileOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseSequenceFileOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseStatsFileOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseGroupDefOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseLowerBoundOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseUpperBoundOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseForbidOrderOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseOrderBoundOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseBlockOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseForbidElementOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseRequireElementOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);
	OptionsList_t::const_iterator ParseSpecialOrderOption(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd);

	std::pair<std::string, OptionsList_t::const_iterator> ExtractDelimetedString(OptionsList_t::const_iterator iterBegin, OptionsList_t::const_iterator iterEnd, const std::string& strOptionCmd, char d1='[', char d2=']');
	int	ExtractNonDelimetedInteger(std::string str, const std::string& strOption);

	std::string CreateDefaultFileName(eFileType type)  const;

	GroupElement*	GetElementFromValues(const std::vector<int>& rValues);


	OptionsList_t	m_ValidOptions;

	bool		m_bCrossNumbers;
	bool		m_bOrders;
	bool		m_bMultiplicity;
	bool		m_bDisplayScreen;
	bool		m_bBlockSearch;

	bool		m_bUseSequenceFile;

	int			m_nLengthLowerBound;
	int			m_nLengthUpperBound;
	int			m_nProgressChangeElement;
	int			m_nBlockLength;

	eDispFormat	m_eDisplayFormat;

	std::string	m_strInFile;
	std::string	m_strSequenceFile;
	std::string m_strStatsFile;

	Set_t		m_RequiredElements;

	std::auto_ptr<FiniteAbelianGroup>	m_apGroup;
	ForbiddenOrderList_t				m_ForbiddenOrders;
	OrderBoundLookup_t					m_OrderBoundLookup;
};

#endif